"""
PyNassi- Struktograammeditor

entwickelt im Rahmen meiner Dilpomarbeit

bentigt: Python ab 2.1, QT ab 2.3, PyQT ab 2.5
getestet auf Windows und Linux
Entwurf: 26.02.2002
Stand:   24.10.2002
i.linkweiler@gmx.de
"""

version='1.2'

# Alles importieren
import sys

from qt import *
try:
  from qtcanvas import *
except ImportError:
  pass
from icons import *
from Statements import *
from commandeditwidget import *
from loopeditwidget import *
from foreditwidget import *
from sourcewidget import *
from struktcanvas import *
from stdoutwidget import *
from setupwidget import *

import options
option=options.Options()
option.laden()

import gettext
import hilfe

if option.get("language")=="EN":
  lang = gettext.translation("messages",".",languages=['en'])
  lang.install()
  hilfe.lng="EN"
elif option.get("language")=="UD":
  lang = gettext.translation("messages",".",languages=['ud'])
  lang.install()
  hilfe.lng="EN"
else:
  gettext.install("messages")

global toplevels

############################
class Struktogramm:
  " 1 Struktogramm besteht aus 1 Struktogramm-Canvas."
  def __init__(self,myworkspace,filename="",assistent=0):
    """
    Erzeuge ein neues Struktogramm im QT-Workspace myworkspace.
    filename=optionaler Dateiname
    assistent=1, das Strukrogramm wird mit dem Assistenten erstellt.
    """
    
    if assistent:
      r=QInputDialog.getText ( _("Neues Struktogramm"),_("Name der ersten Funktion"))
      fnname=str(r[0])

      QMessageBox.about(None, "PyNassi",
        _("""<p>Zum Bearbeiten eines Statements dieses mit der linken Maustaste anklicken.
Zum Einfgen weiterer Funktionen links neben der Funktion klicken.
Zum ndern oder Lschen eines Statements dieses mit der rechten Maustaste anklicken.</p>""" ))

     
      s1=Statement()
      s2=Statement()
      s3=Statement()   
      c1=Command()
      c1.setText(fnname+"()")
      f1=Function()
      f1.setBody(s3)
      f1.setName(fnname)
      s1.setBody(f1)
      s2.setBody(c1)
      bl=Block()
      bl.appendStmt(s1)
      bl.appendStmt(s2)
      rootstmt=Statement()
      rootstmt.setBody(bl)
    else:
      rootstmt=Statement()

    self.workspace=myworkspace
    self.canvas=QCanvas(320,400)
    self.struktcanvas=StruktCanvas(self.canvas,self.workspace)

    self.struktcanvas.set_fatherwindow(self.workspace)
    self.struktcanvas.set_rootstatement(rootstmt)
    self.struktcanvas.set_canvas(self.canvas)

    self.workspace.sublevels.append(self.struktcanvas)
    self.workspace.sublevels[-1].show()  
    self.struktcanvas.draw()
    
    if filename!="":
      self.struktcanvas.laden(filename)

############################################################################
# Es folgt das Basis-GUI
# Bereitstellung des Hauptfensters (Win), bestehend aus:
# Menleiste und Arbeitsbereich (MyWorkspace)
# Der Arbeitsbereich hat eine Liste (sublevels) seiner Unterfenster
############################################################################

class MyWorkspace(QWorkspace):
  "Der Arbeitsbereich des Hauptfensters. Hat n Unterfenster."

  def __init__(self, *args):
    "Erstelle Hauptfenster"
    apply(QWorkspace.__init__,(self,)+args)

    self.setGeometry(40,40,175,75)    
    self.sublevels=[]
    self.setBackgroundPixmap(QPixmap("./icons/backgrnd.png"))

class Win(QMainWindow):
  "Das Programmfenster; hat 1 Men,1 Arbeitsbereich, 1 Statuszeile"

  def __init__(self, *args):
    "Erstelle das Programmfenster"
    apply(QMainWindow.__init__,(self,)+args)

    self.setCaption('PyNassi '+version+' von Inli, Uni DO')
    self.icon=QPixmap("QTlogo.png")
    self.setIcon(self.icon)
    
    self.file=QPopupMenu()

    self.file.insertItem((_("&Neuer Struktogramm-Editor")),self.slotStrktCanvas)
    self.file.insertItem((_("Neues Programmfenster")),self.slotNewMain)
    self.file.insertSeparator()
    self.file.insertItem((_("&Oeffnen")),self.slotLaden)
    self.file.insertItem((_("&Sichern")),self.slotSichern)
    self.file.insertItem((_("Sichern &als...")),self.slotSichernals)
    self.file.insertItem((_("s&chliessen")),self.slotClose)
    self.file.insertItem((_("Drucken")),self.slotPrint)
    self.file.insertSeparator()
    self.file.insertItem((_("Co&de erzeugen")),self.slotCode)
    self.file.insertItem((_("Neues Ausgabefenster")),self.slotAusgabe)
    self.file.insertSeparator()
    self.file.insertItem((_("&Ende")),self.slotQuit)

    self.help=QPopupMenu()
    self.help.insertItem((_("&Information")),self.slotAbout)
    self.help.insertItem((_("&Anleitung")),self.slotHilfe)
    self.help.insertSeparator()
    self.help.insertItem((_("Einstellungen...")),self.slotSetup)

    self.debu=QPopupMenu()
    self.debu.insertItem((_("Code mit &Python ausfhren")),self.slotRun)
    self.debu.insertItem((_("Code mit &Debugger ausfhren")),self.slotDebug)
    self.debu.insertSeparator()
    self.debu.insertItem((_("Debugger Pause/Weiter")),self.slotDebugstop)
    self.debu.insertItem((_("Debugger nchster Schritt")),self.slotDebugstep)
    self.debu.insertSeparator()
    self.debu.insertItem((_("&nur Aufrufe protokollieren")),self.slotDebugmode,0,11)
    #self.debu.insertItem((_("Geschwindigkeit einstellen")),self.slotDebugspeed)
   
    self.menu=QMenuBar(self)
    self.menu.insertItem((_("&Datei")), self.file)
    self.menu.insertItem((_("&Ablauf")), self.debu)
    self.menu.insertItem((_("&Hilfe")), self.help)

    self.menu.show()
 
    self.toolbar=QToolBar(self, _("Datei Operationen"))
    self.tool_new = QToolButton(QPixmap("./icons/strukt.png"),
         _('Neuer Editor'), '',
				 self.slotStrktCanvas,
				 self.toolbar, 'new')
    #self.tool_new.setUsesTextLabel(1)
    self.tool_new.setUsesBigPixmap(1)

    self.tool_open = QToolButton(QPixmap("./icons/fileopenbig.png"),
				 _('Datei ffnen'), '',
				 self.slotLaden,
				 self.toolbar, 'ffnen')
    #self.tool_open.setUsesTextLabel(1)
    self.tool_open.setUsesBigPixmap(1)

    self.tool_close = QToolButton(QPixmap("./icons/filesavebig.png"),
				 _('Sichern in Datei'), '',
				 self.slotSichern,
				 self.toolbar, 'Sichern')
    #self.tool_close.setUsesTextLabel(1)
    self.tool_close.setUsesBigPixmap(1)

    self.tool_print = QToolButton(QPixmap("./icons/print.png"),
				 _('Drucken'), '',
				 self.slotPrint,
				 self.toolbar, 'Print')
    #self.tool_close.setUsesTextLabel(1)
    self.tool_print.setUsesBigPixmap(1)

    
    self.toolbar.addSeparator()

    self.tool_zoom = QToolButton(QPixmap("./icons/zoom32.png"),
				 _('Ansicht vergrssern'), '',
				 self.slotZoom,
				 self.toolbar, 'Zoom')
    self.tool_zoom.setUsesBigPixmap(1)

    self.tool_zoomoff = QToolButton(QPixmap("./icons/zoomoff32.png"),
				 _('Ansicht verkleinern'), '',
				 self.slotZoomoff,
				 self.toolbar, 'Zoom')
    self.tool_zoomoff.setUsesBigPixmap(1)
    self.toolbar.addSeparator()

 
    self.tool_open = QToolButton(QPixmap("./icons/sourcebig.png"),
				 _('Programm-Code aus Strukrogramm erzeugen'), '',
				 self.slotCode,
				 self.toolbar, 'Code erzeugen')
    self.tool_open.setUsesBigPixmap(1)
    self.toolbar.addSeparator()
    
    self.tool_open = QToolButton(QPixmap("./icons/runbig.png"),
				 _('Code mit Python ausfhren'), '',
				 self.slotRun,
				 self.toolbar, 'Ausfhren')
    self.tool_open.setUsesBigPixmap(1)
    self.toolbar.addSeparator()

    self.tool_debug = QToolButton(QPixmap("./icons/debug32.png"),
				 _('Code mit Debugger ausfhren'), '',
				 self.slotDebug,
				 self.toolbar, 'Ausfhren')
    self.tool_debug.setUsesBigPixmap(1)

    self.tool_debugstop = QToolButton(QPixmap("./icons/pause32.png"),
				 _('Debugger Pause/Weiter'), '',
				 self.slotDebugstop,
				 self.toolbar, 'Pause/Cont')
    self.tool_debugstop.setUsesBigPixmap(1)

    self.tool_debugstep = QToolButton(QPixmap("./icons/step32.png"),
				 _('Debugger nchster Schritt'), '',
				 self.slotDebugstep,
				 self.toolbar, 'Step')
    self.tool_debugstep.setUsesBigPixmap(1)
    
    self.tool_debughalt = QToolButton(QPixmap("./icons/halt32.png"),
				 _('Debugger abbrechen'), '',
				 self.slotDebughalt,
				 self.toolbar, 'Break')
    self.tool_debughalt.setUsesBigPixmap(1)

    #self.addToolBar(self.toolbar)
    self.toolbar.show()
    #self.connect(self.toolbar, SIGNAL("clicked(int)"), self.commandCallback)
                             
    self.statusbar=QStatusBar(self)
    label1=QLabel(_("Willkommen im PyNassi Struktogrammeditor!"),self.statusbar)
    label2=QLabel("OK",self.statusbar)

    self.statusbar.addWidget(label1,80)
    self.statusbar.addWidget(label2,20)
 
    label1.show()
    label2.show()
    self.statusbar.show()
    #self.setStatusBar(self.statusbar)

    self.workspace=MyWorkspace(self)
    self.setCentralWidget(self.workspace)

    self.slotAusgabe()
    # self.slotLadeletztes()

    #print "huhu",option.get("language")
    #if option.get("language")=="": 
    #  print "setup..."
    #  self.slotSetup()

   
  def closeEvent(self, QCloseEvent):
    "Ich muss mich schlieen"
    QCloseEvent.accept()
    
  def slotNewMain(self):
    "Neues Programmfenster. Fensterliste mu leider global gehalten werden."
    global toplevels
    toplevels.append(Win())
    toplevels[-1].show()

  def slotStrktCanvas(self):
    """
    Erstelle ein neues Struktogramm bestehend aus einem Statement; und
    Erstelle ein neues Canvas mit diesem Struktogramm  
    """
    Struktogramm(self.workspace,"",1)
  
  def slotClose(self):
    self.close()
    
  def slotQuit(self):
    "Beenden"
    sys.exit()

  def slotAbout(self):
    "Infos ber das Programm"
    QMessageBox.about(self, "PyNassi",
      _("<p>Ein <b>Nassi-Shneiderman-Struktogramm-Editor</b> nach DIN 66261</p>"
    	"<p>Dieses Programm entstand zu Demonstrationszwecken "
    	"im Rahmen meiner Diplomarbeit. </p>"
    	"<p>2002 by i.linkweiler@gmx.de , under GPL </p>" ))

  def slotHilfe(self):
    "Hilfe anzeigen"
    hilfe.zeigen("index.html")
    
  def slotNotImpl(self):
    "Dummy fr nicht impl. Fkt."
    self.statusbar.message("not yet implemented...",1000)
    print "OR: not implemendet yet."

  def slotZoom(self):
    "Neues Fenster mit Sourcecode aus Struktogramm erzeugen"
    topwin=self.workspace.activeWindow()
    if topwin!=None:
      topwin.setZoom(min(topwin.zoom+1,8))

  def slotZoomoff(self):
    "Neues Fenster mit Sourcecode aus Struktogramm erzeugen"
    topwin=self.workspace.activeWindow()
    if topwin!=None:
      topwin.setZoom(1)

  def slotSetup(self):
    w=SetupWidget(self.workspace)
    w.setitem(option)
    self.workspace.sublevels.append(w)
    self.workspace.sublevels[-1].show()

  def slotCode(self):
    "Neues Fenster mit Sourcecode aus Struktogramm erzeugen"
    topwin=self.workspace.activeWindow()
    print topwin
    if topwin!=None:
      code=topwin.makeCode()
    else:
      code="# leer"
    #print code
    self.workspace.sublevels.append(SourceWidget(self.workspace))
    self.workspace.sublevels[-1].show()
    self.workspace.sublevels[-1].setSource(code,topwin)
    self.workspace.sublevels[-1].setName(topwin.myname.replace(".stg",".py"))


  def slotLaden(self):
    "Etwas soll geladen werden..."
    f=QFileDialog.getOpenFileName("./eigene","Struktogramm (*.stg);;Source (*.py);;", self, _("Datei Auswahl"))
    if f.isEmpty()==0:
      f=str(f)
      # passendes Dokument erzeugen und Datei lesen
      if f.count(".stg")>0:
        strukt=Struktogramm(self.workspace,f)
      else:
        self.workspace.sublevels.append(SourceWidget(self.workspace))
        self.workspace.sublevels[-1].show()
        self.workspace.sublevels[-1].laden(f)

  def slotLadeletztes(self):
    "Etwas soll geladen werden..."
    f="fakultaet.stg" #f="runme.py"
    if f!="":
      f=str(f)
      # passendes Dokument erzeugen und Datei lesen
      if f.count(".stg")>0:
        strukt=Struktogramm(self.workspace,f)
      else:
        self.workspace.sublevels.append(SourceWidget(self.workspace))
        self.workspace.sublevels[-1].show()
        self.workspace.sublevels[-1].laden(f)

  def slotAusgabe(self):
    "Ein neues Ausgabefenster"
    self.workspace.sublevels.append(StdoutWidget(self.workspace))
    self.workspace.sublevels[-1].show()
        
  def slotSichern(self):
    "aktuelles Fenster sichern"
    topwin=self.workspace.activeWindow()
    if topwin!=None:  
      try:
        topwin.sichern()
      except AttributeError:
        pass

  def slotSichernals(self):
    "aktuelles Fenster sichern"
    topwin=self.workspace.activeWindow()
    if topwin!=None:
      try:
        topwin.sichern_als()
      except AttributeError:
        pass

  def showDebug(self):
    "ffnen der Datei debug.txt, springe in Codefenster zu Fehlerzeile."
    self.workspace.sublevels.append(SourceWidget(self.workspace))
    self.workspace.sublevels[-1].show()
    self.workspace.sublevels[-1].laden("debug.txt")
    for i in range(1,self.workspace.sublevels[-1].numLines()):
      t=str(self.workspace.sublevels[-1].textLine(i))
      print t
      # Wenn debug.txt ein "Line" enthlt, Springe zu dieser Zeile.
      if t.count("line")==1:
        p=t.index("line")
        print "Fehler in Zeile",i
        line=t[p+5:]
        linenum=""
        for i in line:
          if i.isdigit():
            linenum+= i
        y=eval(linenum)

        topwin=self.workspace.activeWindow()        
        topwin.setActiveWindow()
        topwin.setCursorPosition(y-1,0)

  def slotBadFunction(self):
    "Warnmeldung bei ungltiger Menfunktion"
    QMessageBox.about(self, "PyNassi",_("<p>Diese Funktion arbeitet nur auf Source-Fenstern.</p>") )

  def slotRun(self):
    "Programm des aktuellen Fensters in Python ausfhren"
    topwin=self.workspace.activeWindow()
    if topwin!=None:
      toplevels[0].hide()
      try:
        error=topwin.run()
      except AttributeError:
        error=0
        self.slotBadFunction()
      toplevels[0].show()

  def slotPrint(self):
    "aktuelles Fensters drucken"
    topwin=self.workspace.activeWindow()
    if topwin!=None:
      try:
        error=topwin.drucken()
      except AttributeError:
        error=0
        self.slotBadFunction()

  def slotDebug(self):
    "Programm des aktuellen Fensters im Debugger ausfhren"
    topwin=self.workspace.activeWindow()
    if topwin!=None:
      try:
        topwin.setWorkspace(self.workspace)
        topwin.debug(max(1,option.getvalue("debugspeed")))
      except AttributeError:
        self.slotBadFunction()

  def slotDebugstop(self):
    "Debugger anhalten"
    topwin=self.workspace.activeWindow()
    if topwin!=None:
      try:
        topwin.pause()
      except AttributeError:
        self.slotBadFunction()

  def slotDebugstep(self):
    "Debugger nchter Schritt"
    topwin=self.workspace.activeWindow()
    if topwin!=None:
      try:
        topwin.step()
      except AttributeError:
        self.slotBadFunction()

  def slotDebughalt(self):
    "Debugger anhalten"
    topwin=self.workspace.activeWindow()
    if topwin!=None:
      try:
        topwin.halt()
      except AttributeError:
        self.slotBadFunction()

  def slotDebugmode(self):
    "Debuggermodus einstellen"
    topwin=self.workspace.activeWindow()
    if topwin!=None:
      try:
        topwin.toggleDebugmode()
      except AttributeError:
        self.slotBadFunction()
    self.debu.setItemChecked ( 11, topwin.debugmode)

  def slotDebugspeed(self):
    "Debugger GEschwindigkeit setzen"
    topwin=self.workspace.activeWindow()
    if topwin!=None:
      try:
        topwin.setDebugspeed()
      except AttributeError:
        self.slotBadFunction()

    
  def commandCallback(self, item):
    "Auf QT-QUIT reagieren."
    if item==TB_QUIT:    
      self.slotQuit()
  
def main():
  "1 Hauptfenster erzeugen, und auf Ende warten."
  global toplevels
  app=QApplication(sys.argv)
  
  toplevels=[Win()]
  toplevels[0].show()
  app.connect(app, SIGNAL('lastWindowClosed()'), app, SLOT('quit()'))

  app.exec_loop()
  toplevels=[]

if __name__=="__main__":
  main()
