"""
Ein Widget zum Bearbeiten von Text, insbes. Sourcecode.
Erweitert QMultiLineEdit um laden, sichern, ausfhren u.a.
21.10.02 /il
"""

import sys
import os
try:
  import win32api
except:
  pass
from qt import *
from debugger import *
from infowidget import *

class SourceWidget(QMultiLineEdit):
  """
  Ein Widget zum Bearbeiten von Text, insbes. Sourcecode.
  Erweitert QMultiLineEdit um laden, sichern, ausfhren u.a.
  """
  def __init__(self,*args):
    "Erzeuge das Widget"
    apply(QMultiLineEdit.__init__,(self,) + args)
    self.setMinimumSize(256,256)
    self.resize(320,256)
    self.icon=QPixmap("./icons/find.xpm")
    self.setIcon(self.icon)
    self.setName("noname.py")
    self.timer=QTimer(self)
    self.connect( self.timer, SIGNAL("timeout()"), self.timeout ) 
    self.myStruktogramm=None
    self.myInfoWidget=None
    self.debuggerRunning=0
    self.debugmode=0
    self.debugspeed=2
    
    font=QFont("Courier",10)
    self.setFont(font)

    #self.setGeometry(QRect(170,70,260,271))
    self.setHighlightColor(QColor(Qt.red))

  def drucken(self):
    Margin = 10
    pageNo = 1
    printer=QPrinter()
    if printer.setup(self):
      p = QPainter()
      p.begin(printer)
      yPos = 0
      fm = p.fontMetrics()
      metrics = QPaintDeviceMetrics(printer)
      for i in range(self.numLines()):
        if yPos > metrics.height():
          pageNo = pageNo + 1
          printer.newPage()
          yPos = 0

        p.drawText(Margin,Margin + yPos,metrics.width(),fm.lineSpacing(),Qt.ExpandTabs | Qt.DontClip,self.textLine(i))
        yPos = yPos + fm.lineSpacing()
      p.end()

  def setHighlightColor(self,c):
    "Setze die Text-Markierungsfarbe auf QColor c"
    cg=self.colorGroup()
    cg.setColor(QColorGroup.Highlight,c)
    # print cg
    pal = QPalette()
    pal.setActive(cg)
    pal.setInactive(cg)
    self.setProperty('palette',QVariant(pal))
  
  def setWorkspace(self,ws):
    "Setzte workspace"
    self.workspace=ws
    
  def timeout( self ):
    """
    Timer-Methode. Auslesen der Meldungen des Debuggers aus globaler tracelist
    und Weitergeben an das eigene Info-Widget
    """
    global tracelist

    # Ausgabe und Auswertung der Trace-Meldungen
    if tracelist!=[]:
      t=tracelist.pop(0)
            
      zeile=t[0]-2  #-2 mu sein
      
      s=""
      if zeile>=0:
        try:
          s=str(self.textLine(zeile))
          if len(s)>20:
            # Ausgabe ggf. krzen
            s=s[:18]+"..."
        except:
          pass

      if self.debugmode==0 or t[2]!="line":
        self.myInfoWidget.addItem(t[2],str(zeile)+" "+s+" "+t[1])

      if zeile<-500:
        self.timer.stop()
        print "Timer beendet."
        self.debuggerRunning=0
      else:
        self.setCursorPosition( zeile, 0, 0 ) 
        self.setCursorPosition( zeile, 80, 1 ) 
        if self.myStruktogramm!=None:
          self.myStruktogramm.setMarker(zeile)

  def closeEvent(self,ce):
    "Fenster wurde geschlossen, Timer also stoppen."
    print "stopping timer"
    self.timer.stop()
    ce.accept()
 
  def slotClose(self):
    self.close(1)

  def setSource(self,src,myStg=None):
    """
    Setze Sourcecodeliste als Text ins Editfeld,
    und merke zugehriges Struktogramm
    """
    self.setText(src)
    self.myStruktogramm=myStg

  def setName(self,n):
    "Setze den Dateinamen des Fensters"
    self.myname=n
    self.setCaption(_('Quelltext (%s)') %self.myname )

  def laden(self,filename=""):
    "laden einer Datei in den Editor"
    if filename!="":
      self.setName(filename)     
    handle=open(self.myname,"r")
    t=handle.read()
    handle.close()
    self.setText(unicode(t,'cp437'))

  def sichern(self):
    "sichern des aktuellen Editorinhalts"
    if self.myname=="":
      self.myname="noname.txt"
    print "sichere...",self.myname
    uc=self.text()
    try:
      t=str(uc)
    except:
      try:
        t=unicode(self.text()).encode("cp437", "replace")
      except:
        t="invalid char in string"  

    handle=open(self.myname,"w")
    handle.write(t)
    handle.close()

  def sichern_als(self):
    "Auswahl eines neuen Dateinamens, dann sichern"
    f=QFileDialog.getSaveFileName(self.myname,"Source (*.py)", self, _("Datei Auswahl"))
    if f.isEmpty()==0:
      self.setName(str(f))
      self.sichern()
   
  def run(self):
    """
    Source an Python/Shell bergeben
    Rckgabe: >0, wenn Fehler auftraten. Fehlertext in debug.txt   
    """
    self.sichern()
    if os.name=="nt" or os.name=="win":
      prg='"'+self.myname.replace("/","\\")+'"'
      os.system("command /C python -i "+prg)
    else:
      prg='"'+self.myname+'"'
      os.system("python -i "+prg) 
    r=0
    try:
      handle=open("debug.txt","rb")
      handle.seek(0,2)
      r=handle.tell()
      handle.close()
    except IOError:
      pass
    return r

  def pause(self):
    "Timer anhalten"
    if self.timer.isActive():
      self.timer.stop()
    else:
      self.timer.start(self.debugspeed*100)

  def step(self):
    "Timer 1* starten"
    self.timer.start(50,1)

  def halt(self):
    "Break an Debugger senden"
    breaklist.append(1)

  def debug(self,speed=4):
    """
    Source ausfhren im internen Debugger.
    Rckgabe: Tupel (Fehlerzeile,Fehlertext)
    """
    #self.workspace.sublevels.append(InfoWidget(self.workspace))
    if self.debuggerRunning:
      print "Debugger schon aktiv."
      return
    
    if self.myInfoWidget==None:
      self.myInfoWidget=InfoWidget(self.workspace)
    self.myInfoWidget.show()

    result=(0,"")

    self.debugspeed=speed
    self.timer.start( self.debugspeed*100 )         # emit signal every 200 mseconds 
    self.sichern()

    prg=unicode(self.text()).encode("cp437", "replace")
    # prg=str(self.text())
    d=Debugger(prg,0)
    debugthread=threading.Thread(target=d.run)
    debugthread.start()

    self.debuggerRunning=1

    return result

  def toggleDebugmode(self):
    "Schalte den Debugmode um"
    self.debugmode=not self.debugmode

  def setDebugspeed(self):
    "Setzte die Debugger Geschwindigkeit"
    r=QInputDialog.getInteger (
      _("Debugger Geschwindigkeit"),
      _("Eingabe\n1=Schnell ... 100=Langsam."),
      self.debugspeed, 1, 100, 1 )
    self.debugspeed=r[0]
    self.timer.start( self.debugspeed*100 )

def main():
    "Zum Test:"

    a = QApplication(sys.argv)
    QObject.connect(a,SIGNAL('lastWindowClosed()'),a,SLOT('quit()'))
    w = SourceWidget()
    a.setMainWidget(w)  
    w.show()
    a.exec_loop()

if __name__=="__main__":
  main()
