"""
Der Struktogrammeditor.
Stellt ein Struktogramm in einem Canvas dar und erlaubt Bearbeitung.
Stand 05.10.2002
"""

import sys
import os
import pickle
import copy

from qt import *
try:
  from qtcanvas import *
except ImportError:
  pass
from icons import *
from Statements import *
from commandeditwidget import *
from loopeditwidget import *
from sourcewidget import *
from foreditwidget import *
from functioneditwidget import *
from klasseneditwidget import *
from ausgabeeditwidget import *
from eingabeeditwidget import *

class StruktCanvas(QCanvasView):
  """
  Der Struktogrammeditor.
  Stellt ein Struktogramm in einem Canvas dar und erlaubt Bearbeitung.
  """
  def __init__(self,*args):
    "Erzeuge ein neues StrukrogrammCanvas"
    apply(QCanvasView.__init__,(self,) + args)
    self.myname="./eigene/noname.stg"
    self.set_titel()
    self.icon=QPixmap("./icons/strukt.png")
    self.setIcon(self.icon)
    self.setFocusPolicy(2)
    self.menulock=0
    self.zoom=1
    self.marker=None
    self.poshash={}
    self.resize(320,400)
    self.items=[]
    self.mauspos=(0,0)

  def set_titel(self, dateiname=""):
    self.setCaption(_('Struktogramm (')+self.myname+")")   

  def set_rootstatement(self, sm):
    "Setze das Wurzel-Statement"
    self.rootstatement=sm

  def set_canvas(self, mycanvas):
    "Setze das Canvas."
    self.canvas=mycanvas
    self.w=320
    self.h=200

  def setZoom(self, neuzoom):
    "Stelle den Zoom-Faktor ein"
    self.zoom=neuzoom
    print "Zoom=",neuzoom
    self.paintEvent(0)
    self.marker=None

  def set_fatherwindow(self, vater):
    "Setzte den Workspace, in dem das Canvas dargestellt wird."
    self.workspace=vater

  def sichern(self,dateiname=""):
    """
    Sichere das Struktogramm.
    Pickling erlaubt hier einfaches Speichern der gesamten Objekt-Struktur
    beginnend ab dem Wurzelstatement.
    """
    
    if self.myname=="":
      self.myname="noname.stg"
    if self.myname.count(".stg")==0:
      self.myname += ".stg"
    print "sichere..."

    f=open(self.myname,"w")
    p=pickle.Pickler(f)
    p.dump(self.rootstatement)
    f.close()
    self.set_titel()

  def sichern_als(self,dateiname=""):
    "Sichern des Struktogramms mit Dateiauswahl"
    f=QFileDialog.getSaveFileName(self.myname,"Struktugramm (*.stg)", self, "Datei Auswahl")
    if f.isEmpty()==0:
      self.myname=str(f)
      self.sichern()
      self.set_titel()
    
  def laden(self,filename=""):
    "Lade ein neues Struktogramm und setzte es als Wurzel."
    if filename!="":
      self.myname=filename
    f=open(self.myname,"r")
    self.rootstatement=pickle.Unpickler(f).load()
    f.close() 
    self.set_titel()
    self.draw()

  def auswahl(self, poptext,pos):
    "Stellt ein POPUP-Menu dar."
    if self.menulock==0:
      # self.menulock=1  # Doppelte Popups durch GUI verhindern
      popup=QPopupMenu(self.workspace,"BlaBla")   
      id=1
      for i in poptext:
        popup.insertItem(i,id)
        id=id+1
      r=popup.exec_loop(pos)
      self.menulock=0
    else:
      r=-1
    return r

  def drucken(self):
    print "Drucke..."
    printer=QPrinter()
    stmt=self.rootstatement
       
    print printer.printerName()
    printer.setup()
    printer.setDocName("PyNassi_"+self.myname)

    printerinf=QPaintDeviceMetrics(printer)
    w,h = printerinf.width(), printerinf.height()

    painter=QPainter(printer)
    painter.drawLine(0,h/32,w,h/32)
    painter.drawText(w/4,h/48,"Struktogramm "+self.myname)
    stmt.draw(painter,w/10,h/20,w*0.8,h*0.9)   
    painter.end()

    printer.newPage()
    print "OK"

  def bearbeiten(self, body):
      "ffnet das passende Edit-Fenster zum Statement <body>"
      bodyname=body.__class__.__name__  
      if bodyname=="Command" or bodyname=="Exitcommand":
        EditW=CommandeditWidget(self.workspace)
        EditW.setitem(body)
        EditW.setexitcmd(self.hideanddraw)
        self.workspace.sublevels.append(EditW)   
        self.workspace.sublevels[-1].show()
      elif bodyname=="Loop" or bodyname=="Ifthen":
        EditW=LoopeditWidget(self.workspace)
        EditW.setitem(body)
        EditW.setexitcmd(self.hideanddraw)
        self.workspace.sublevels.append(EditW)   
        self.workspace.sublevels[-1].show()
      elif bodyname=="Forloop":
        EditW=ForeditWidget(self.workspace)
        EditW.setitem(body)
        EditW.setexitcmd(self.hideanddraw)
        self.workspace.sublevels.append(EditW)   
        self.workspace.sublevels[-1].show()
      elif bodyname=="Function":
        EditW=FunctioneditWidget(self.workspace)
        EditW.setitem(body)
        EditW.setexitcmd(self.hideanddraw)
        self.workspace.sublevels.append(EditW)   
        self.workspace.sublevels[-1].show()
      elif bodyname=="Klasse":
        EditW=KlasseneditWidget(self.workspace)
        EditW.setitem(body)
        EditW.setexitcmd(self.hideanddraw)
        self.workspace.sublevels.append(EditW)   
        self.workspace.sublevels[-1].show()
      elif bodyname=="Ausgabe":
        EditW=AusgabeeditWidget(self.workspace)
        EditW.setitem(body)
        EditW.setexitcmd(self.hideanddraw)
        self.workspace.sublevels.append(EditW)   
        self.workspace.sublevels[-1].show()
      elif bodyname=="Eingabe":
        EditW=EingabeeditWidget(self.workspace)
        EditW.setitem(body)
        EditW.setexitcmd(self.hideanddraw)
        self.workspace.sublevels.append(EditW)   
        self.workspace.sublevels[-1].show()

      elif bodyname=="Block":
        s1=Statement()
        fn=Function()
        fn.setBody(s1)

        poptext=[_("Funktion hinzufgen"),
                 _("Statement unten hinzufgen"),
                 "print"]
        x=self.auswahl(poptext,self.mauspos)
        if x==1:
          s2=Statement()
          s2.setBody(fn)
          body.insertStmt(s2)
        elif x==2:
          body.appendStmt(s1)
        elif x==3:
          self.drucken()
        self.marker=None
      else:
        print "Unbekannter Typ:",bodyname       

  def contentsMousePressEvent(self, e):
    "Es wurde ein Mausklick im Struktogramm festgestellt."
    button=e.button()
    x=e.pos().x()
    y=e.pos().y()
    self.mauspos=e.globalPos()
    
    # Suche das angeklickte Statement beginnend ab der Wurzel
    klickedStmt=self.rootstatement.find(x,y)
    item=klickedStmt[-1]
    self.marker=item.pos

    # Suche auch das evtl. bergeordnete Block-Statement
    blockitem=None
    for i in klickedStmt:
      name=i.__class__.__name__
      if name=="Block":
        blockitem=i

    self.draw
    
    # Einiges vorbereiten...
    s1=Statement()
    s2=Statement()
    c1=Command()
    b1=Block()
    b1.appendStmt(s1)
    b1.appendStmt(s2)
    l1=Loop()
    l1.setBody(s1)
    i1=Ifthen()
    i1.setBodytrue(s1)
    i1.setBodyfalse(s2)
    f1=Forloop()
    f1.setBody(s1)
    e1=Exitcommand()
    fn=Function()
    fn.setBody(s1)
    cl1=Klasse()
    cl1.setBody(s1)
    aus1=Ausgabe()
    ein1=Eingabe()
    
    name=item.__class__.__name__     
    body=item.body  

    # Eigenschaften des gefundenen Elements bearbeiten.    
    if button==1 and body!=None:     # Linksklick auf Statement=>Bearbeiten
      self.bearbeiten(body)

    elif button==1:             # Rechtsklick oder leeres Statement: Statementtyp ndern.

      print "ist ein ",name        
        
      if name=="Statement":
        print "huhu"
        poptext=[_("-> Anweisung"),
                 _("-> Block"),
                 _("-> while-Schleife"),
                 _("-> if..else"),
                 _("-> for-Schleife"),
                 _("-> return-Anweisung"),
                 _("-> Funktionsdefinition"),
                 _("-> Klassendefinition"),
                 _("-> Ausgabe"),
                 _("-> Eingabe"),
                 _("lschen")]
                 
        x=self.auswahl(poptext,e.globalPos())
        if x==1:
          item.body=c1
        elif x==2:
          item.body=b1
        elif x==3:
          item.body=l1
        elif x==4:
          item.body=i1
        elif x==5:
          item.body=f1
        elif x==6:
          item.body=e1
        elif x==7:
          item.body=fn
        elif x==8:
          fn.setName("__init__")
          sm=Statement()
          sm.setBody(fn)
          bl1=Block()
          bl1.appendStmt(sm)
          sm1=Statement()
          sm1.setBody(bl1)
          cl1=Klasse()
          cl1.setBody(sm1)
          item.body=cl1
        elif x==9:
          item.body=aus1
        elif x==10:
          item.body=ein1
        elif x==11:
          item.body=None

        print "Auswahl:",x
      self.marker=None

    elif button==2 and blockitem!=None: #Rechtsklick auf Block
      poptext=[_("Statement unterhalb hinzufgen"),
               _("Statement oberhalb einfgen"),
               _("Statement lschen")]
      x=self.auswahl(poptext,self.mauspos)
      if x==1:
        if not blockitem.insertbelow(item,s1):
          if name=="Statement":
            olditem=copy.deepcopy(item)
            block=Block()
            block.appendStmt(olditem)         
            block.appendStmt(s1)
            item.body=block
      elif x==2:
        blockitem.insertabove(item,s1)
      elif x==3:
        item.body=None # Sicherheitshalber auch den Body lschen...
        blockitem.removeStmt(item)
      self.marker=None

    self.draw()

  def paintEvent(self,qe):
    "Das Canvas aktualisieren"
    if self.w<>self.width()-4:
      self.marker=None

    self.w,self.h=self.width()-4,self.height()-4
    self.canvas.resize(self.w,self.h*self.zoom)
    self.draw()

  def hideanddraw(self):
    "Den Marker deaktivieren, dann draw"
    self.marker=None
    self.draw()

  def draw(self):
    "Das Canvas aktualisieren"
    for i in self.items:
      i.hide()
      
    self.items=[]
    self.items.extend(self.rootstatement.out(self.canvas,0,0,self.w,self.h*self.zoom))
    if self.marker!=None:
      pos=self.marker
      self.items.append(QCanvasRectangle ( pos[0]+1,pos[1]+1,pos[2]-2,pos[3]-2, self.canvas ) )
      self.items[-1].setPen(QPen(QColor(255,0,0)))
    for i in self.items:
      i.show()
    self.canvas.update() 

  def setMarker(self,zeile):
    "zeichne Markierung zur Programmzeile"
    if self.poshash.has_key(zeile):
      self.marker=self.poshash[zeile].pos
    else:
      self.marker=None
    if self.isVisible:
      self.draw()

  def makeCode(self):
    """
    Erzeugt Source aus dem Struktogramm.
    in poshash wird das zur Programmzeile gehrende Struktogrammobjekt eingetragen. 
    Rckgabe: Liste von(Sourcezeile,Position im Canvas)
    """
    sourcelist=[]
    self.rootstatement.make_Pycode(0,sourcelist)

    self.poshash={}
    zeile=0

    r=""
    for i in sourcelist:
      r+= i[0]+NL
      zeilenzahl=i[0].count(NL)+1
      for j in range(zeilenzahl):
        self.poshash[zeile]=i[1]  
        zeile += 1

    return r

def main():
    "Zum Test:"
    s1 = Statement()

    a = QApplication(sys.argv)
    QObject.connect(a,SIGNAL('lastWindowClosed()'),a,SLOT('quit()'))
    canvas=QCanvas(320,200)
    
    w = StruktCanvas(canvas)
    w.set_rootstatement(s1)
    w.set_canvas(canvas)
    w.set_fatherwindow(w)
    
    a.setMainWidget(w)
    
    w.show()
    a.exec_loop()
    print "Ende"

if __name__=="__main__":
  def _(x): return x
  import __builtin__
  __builtin__.__dict__['_'] = _
  main()
